#include "RotateTransform.h"
#include "iostream"
#include <math.h>

RotateTransform::RotateTransform(std::string name):TransformElement(name){
	this->axis = UNDEFINE;
	this->isEnable = true;

	this->theta = new RangedDouble();
}

RotateTransform::RotateTransform(AXIS axis,std::string name):TransformElement(name){
	this->axis = axis;
	this->isEnable = true;
	this->r = 10;
	this->theta = new RangedDouble();
}
RotateTransform::RotateTransform(AXIS axis,double theta,std::string name):TransformElement(name){
	this->axis = axis;
	this->isEnable = true;
	this->r = 10;
	this->theta = new RangedDouble();
	this->theta->SetValue(theta);
}

char RotateTransform::GetAxisNameFromEnum(RotateTransform::AXIS axis){
	switch(axis){
	case RotateTransform::X:
		return 'X';
	case RotateTransform::Y:
		return 'Y';
	case RotateTransform::Z:
		return 'Z';
	default:
		return '@';
	}
}


void RotateTransform::Dump(int depth,std::stringstream &ret){
	std::string padding = MyUtility::PaddingChar(depth,' ');
	ret << padding << "RotateTransform" << std::endl;
	padding = MyUtility::PaddingChar(depth+2,' ');
	ret << padding << "Name = "<< this->name << std::endl;

	if(this->isEnable){
		ret << padding<< "Transform is Enable.\n";
	}else{
		ret << padding<< "Transform is Disable.\n";
	}

	if(this->axis == RotateTransform::X){
		ret << padding<< "Rotation axis is X.\n";
	}else if(this->axis == RotateTransform::Y){
		ret << padding<< "Rotation axis is Y.\n";
	}else if(this->axis == RotateTransform::Z){
		ret << padding<< "Rotation axis is Z.\n";
	}else{
		ret << padding<< "Rotation axis is undefined.\n";
	}

	ret << padding<< "Rotation angle is "<< this->theta->GetValue() <<".\n";
}


void RotateTransform::Dump(std::stringstream &ret){
	this->Dump(0,ret);
}

void RotateTransform::Dump(void){
	std::stringstream ss;
	this->Dump(ss);
	std::cout<<ss.str();
}

void RotateTransform::Apply(void){
	if(this->isEnable){
		double theta;

		theta = this->theta->GetValue();
		if(this->isNegative){
			theta = -theta;
		}
		switch(this->axis){
			case RotateTransform::X:
				glRotated(theta,1,0,0);
				break;
			case RotateTransform::Y:
				glRotated(theta,0,1,0);
				break;
			case RotateTransform::Z:
				glRotated(theta,0,0,1);
				break;
		}
	}
}
void RotateTransform::SetValue(void){
	std::cerr << "Ă񂶂Ⴞ\n";
	
}

void RotateTransform::Draw(double r){
	double temp;

	temp = this->r;

	this->r = r;

	this->Draw();

	this->r = temp;

}
void RotateTransform::Draw(void){
	double degree;
	double sgn;

	glDisable(GL_LIGHTING);
	if(this->theta > 0){
		sgn = 1;
	}else{
		sgn = -1;
	}
				

	switch(this->axis){
		case RotateTransform::X:
			glColor3d(1.0f,0.0f,0.0f);
			glBegin(GL_LINE_STRIP);
			if((this->theta->GetValue() < 10.0) && (this->theta->GetValue()  > -10.0)){
				glVertex3d(0,r*cos(0.0),r*sin(0.0));
				glVertex3d(0,r*cos(sgn*(this->theta->GetValue())*(3.14/180)),r*sin(sgn*(this->theta->GetValue())*(3.14/180)));
			}else{
				for(degree = 0;degree < sgn*theta->GetValue() ; degree += 10){
					glVertex3d(0,r*cos(sgn*degree*(3.14/180)),r*sin(sgn*(degree)*(3.14/180)));
					glVertex3d(0,r*cos(sgn*(degree+10)*(3.14/180)),r*sin(sgn*(degree+10)*(3.14/180)));
				}
				glVertex3d(0,r*cos(this->theta->GetValue())*(3.14/180),r*sin(this->theta->GetValue()*(3.14/180)));
			}
			break;
		case RotateTransform::Y:
			glColor3d(0.0f,1.0f,0.0f);
			glBegin(GL_LINE_STRIP);
			if((this->theta->GetValue() < 10) && (this->theta->GetValue()  > -10)){
				glVertex3d(r*cos(0.0),0,r*sin(0.0));
				glVertex3d(r*cos(sgn*(this->theta->GetValue())*(3.14/180)),0,r*sin(sgn*(this->theta->GetValue())*(3.14/180)));
			}else{
				for(degree = 0;degree < sgn*theta->GetValue() ; degree += 10){
					glVertex3d(r*cos(sgn*degree*(3.14/180)),0,r*sin(sgn*(degree)*(3.14/180)));
					glVertex3d(r*cos(sgn*(degree+10)*(3.14/180)),0,r*sin(sgn*(degree+10)*(3.14/180)));
				}
				glVertex3d(r*cos(this->theta->GetValue())*(3.14/180),0,r*sin(this->theta->GetValue()*(3.14/180)));
			}
			break;
		case RotateTransform::Z:
			glColor3d(0.0f,0.0f,1.0f);
			glBegin(GL_LINE_STRIP);
			if((this->theta->GetValue() < 10.0) && (this->theta->GetValue()  > -10.0)){
				glVertex3d(r*cos(0.0),r*sin(0.0),0);
				glVertex3d(r*cos(sgn*(this->theta->GetValue())*(3.14/180)),r*sin(sgn*(this->theta->GetValue())*(3.14/180)),0);
			}else{
				for(degree = 0;degree < sgn*theta->GetValue() ; degree += 10){
					glVertex3d(r*cos(sgn*degree*(3.14/180)),r*sin(sgn*(degree)*(3.14/180)),0);
					glVertex3d(r*cos(sgn*(degree+10)*(3.14/180)),r*sin(sgn*(degree+10)*(3.14/180)),0);
				}
				glVertex3d(r*cos(this->theta->GetValue())*(3.14/180),r*sin(this->theta->GetValue()*(3.14/180)),0);
			}
			break;
	}
	glEnd();
	glDisable(GL_LIGHTING);

}


void RotateTransform::SetValue(double theta){
	this->theta->SetValue(theta);
}

void RotateTransform::GetValue(void){
	std::cerr << "Ă񂶂Ⴞ\n";
}

void RotateTransform::GetValue(double *retValue){
	*retValue = this->theta->GetValue();
}

RotateTransform::~RotateTransform(){
}
